# -*- coding: utf-8 -*-
# udp.py.py
#
# Coded by: Koji KITANO <koji.kitano52@gmail.com>,
# Human Systems Laboratory, Department of Systems Science,
# Graduate School of Informatics, Kyoto University.


import socket
from threading import Thread

import numpy as np
import pyaudio # 音声入出力ライブラリ

from pylab import *
import time




class UDP():
    def __init__(self):
        # 送信するデータを格納する配列
        self.frames = []

        # ソケットモジュール
        self.udp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        # 終了フラグ
        self.finFlag = False

    def getJoinedData(self):
        return b"".join(self.frames) #CHUNKごとにframesに格納されているデータを結合

    def getData(self):
        return np.frombuffer(self.getJoinedData(), dtype = "int16") / float(2**15) #録音データがstringなので数値データに戻す


class Server(Thread, UDP):
    def __init__(self, CHUNK):
        Thread.__init__(self)
        UDP.__init__(self)

        # メインのスレッドが終了したときにサブスレッドも終了
        self.setDaemon(True)

        self.udp.bind(("127.0.0.1", 12345))

        # チャンクサイズ（一度に読み込むデータの数）
        self.CHUNK = CHUNK * 4 #ここ重要！

        self.moveFlag = True

    def run(self):
        while True:
            try:
                # 指定したチャンクサイズのデータをソケットから受信する
                # recvは受信したデータ(bytes)，addrはソケットのアドレス
                recv, addr = self.udp.recvfrom(self.CHUNK)

                # 受信したデータを格納
                self.frames.append(recv)

                if self.moveFlag:
                    # 受信終了の為のタイムアウト時間を設定
                    self.udp.settimeout(1) # 1秒でタイムアウト
                    self.moveFlag = False

            except socket.error as err:
                # タイムアウト or エラーでループ脱出
                break

        self.finFlag = True # 終了フラグ

class Client(Thread, UDP):
    def __init__(self, stream, CHUNK, recTime):
        Thread.__init__(self)
        UDP.__init__(self)

        # メインのスレッドが終了したときにサブスレッドも終了
        self.setDaemon(True)

        # チャンクサイズ（一度に読み込むデータの数）
        self.CHUNK = CHUNK

        # 録音するためのストリーム
        self.stream = stream

        # 録音時間
        self.recTime = recTime

    def run(self):

        for i in range(int(self.stream._rate / self.CHUNK * self.recTime)):
            # recTimeの時間だけ録音
            self.frames.append(self.stream.read(self.CHUNK))


        self.finFlag = True # 録音終了フラグ

        for frame in self.frames:
            self.udp.sendto(frame, ("127.0.0.1", 12345)) # データ送信

if __name__ == "__main__":
    FORMAT = pyaudio.paInt16
    CHANNELS = 2
    RATE = 44100
    CHUNK = 1024
    RECORD_SECONDS = 10

    p = pyaudio.PyAudio()

    stream = p.open(format = FORMAT,
                    channels = CHANNELS,
                    rate = RATE,
                    input = True,
                    frames_per_buffer = CHUNK,
                    )

    client = Client(stream, CHUNK, RECORD_SECONDS)
    server = Server(CHUNK)
    server.start()
    client.start()

    while not client.finFlag:
        time.sleep(0.1)
    else:
        Cresult = client.getData() #録音終了を待ち、数値データにして取得

    while not server.finFlag:
        time.sleep(0.1)
    else:
        Sresult = server.getData() #データ受信終了を待ち、数値データにして取得

    print ("Before sample:", len(Cresult)) #送信前のサンプル数
    print ("After sample:", len(Sresult) ) #送信後のサンプル数
    print (float(len(Sresult))/len(Cresult)*100, "% data could be send") #何％送れたか
