# -*- coding: utf-8 -*-
# main.py
#
# Coded by: Koji KITANO <koji.kitano52@gmail.com>,
# Human Systems Laboratory, Department of Systems Science,
# Graduate School of Informatics, Kyoto University.

import sys
import time
import datetime

from get_weather import *
from get_geolocation import *

from serial_to_cotton import Serial2Cotton
from tcp_to_raspi import TCP2Raspi
from talker import Talker

if __name__ == '__main__':

    """ 初期化処理 """
    # シリアル通信でコットンと通信
    cotton = Serial2Cotton()

    # TCP通信でラズパイと接続
    raspi = TCP2Raspi(sys.argv[1], 6666)

    # 自分側のデータを格納する辞書
    own_data = {'light_level' : 0, 'people_sense': 0, 'open': 0, 'weather':0, 'time':0}

    # 天気APIを叩いて辞書に登録
    location = get_geolocation()
    own_data['weather'] = get_weather(*location)
    # API叩いてからの経過時間を計測開始
    api_time = time.time()

    # 相手側のデータを格納する辞書
    his_data = {'light_level' : 0, 'people_sense': 0, 'open': 0, 'weather':0, 'time':0}

    # コットンの処理を待つ
    time.sleep(2)

    """ 通信開始 """

    # コットンに初期値を送る
    print("serial sending0")
    cotton.send_data(own_data)

    while True:

        """ 自分側データの更新 """
        # コットンからデータを受取って更新
        print("serial sending")
        own_data = cotton.read_data()
        print("own_data")
        exit()

        # 前回APIを叩いた時から30秒経過していたらAPIを叩く
        if (time.time() - api_time) > 30:
            # 地理情報APIを叩く
            location = get_geolocation()
            # 天気APIを叩いて辞書に登録
            own_data['weather'] = get_weather(*location)
            # 最後に叩いた時間を更新
            api_time = time.time()

        # 現在時刻を取得して辞書に登録
        own_data['time'] =  datetime.datetime.now().hour

        """ 相手側データを更新 """
        # データを相手側のラズパイと交換
        his_data = raspi.exchange_data(own_data)
        print("exchanged")

        # コットンにデータを送る
        cotton.send_data(his_data)
        print("serial sended")

        """ 通話 """
        if (own_data['open']==1)and(his_data['open']==1):
            # もし両方の窓が空いているならば通話開始
            talker = Talker(sys.argv[1], 40007)

            print("start calling")
            talker.stop()

            while True:
                if (cotton.serial.read(1).decode() != 's'):
                    break
            print("stop calling")
            talker.stop()
