# -*- coding: utf-8 -*-
# serial_to_cotton.py
#
# Coded by: Koji KITANO <koji.kitano52@gmail.com>,
# Human Systems Laboratory, Department of Systems Science,
# Graduate School of Informatics, Kyoto University.


import serial

class Serial2Cotton():
    def __init__(self,dev_name='/dev/serial0'):
        # シリアル通信を開始する
        self.serial = serial.Serial(dev_name, 9600, timeout=None)

    def read_data(self):
        # コットンから送られてきたデータを一行ぶん取り出す
        data = self.serial.readline().decode('utf-8').rstrip('\r\n')
        print('have' + data)

        # "照度, 測距, 窓の開閉"という形で格納されている
        # コンマ区切りでデータを分けたのち，辞書に格納
        data = data.split(',')

        data_dict['light_level'] = int(data[0])
        data_dict['people_sense'] = int(data[1])
        data_dict['open'] = int(data[2])

        return data_dict

    def send_data(self, data=None, calling=0):
        """ データをcottonに送る """

        if data is None:
            data_str = '0,0,0,0,0'
        else:
            data_str =  str(data['light_level']) \
                + ',' + str(data['people_sense']) \
                + ',' + str(data['open']) \
                + ',' + str(data['weather']) \
                + ',' + str(data['time'])

        self.serial.write(data_str.encode('utf-8'))

if __name__ == '__main__':
    import time
    dev_name = '/dev/serial0'
    cotton = Serial2Cotton(dev_name)

    data_dict = {} #{'light_level': 12, 'people_sense': 13,'open': 14}

    while True:
        #cotton.send_data()
        cotton.send_data()
        data_dict = cotton.read_data()
        print(data_dict)
        time.sleep(0.3)

    """
    while True:
        order = 'r'.encode('utf-8')
        ser.write(order)
        time.sleep(1)
        order = 'g'.encode('utf-8')
        ser.write(order)
        time.sleep(1)
        order = 'b'.encode('utf-8')
        ser.write(order)
        time.sleep(1)
    """
