# -*- coding: utf-8 -*-
# talker.py
#
# Coded by: Koji KITANO <koji.kitano52@gmail.com>,
# Human Systems Laboratory, Department of Systems Science,
# Graduate School of Informatics, Kyoto University.


# TODO: 再生用ストリームと録音用ストリームは別である必要があるか？

import socket
from threading import Thread

from multiprocessing import Process, Manager, Queue

import numpy as np
import pyaudio # 音声入出力ライブラリ

import time



FORMAT = pyaudio.paInt16
CHANNELS = 1
RATE = 44100
CHUNK = 1024 * 30

# 再生待ちバッファのサイズ
BUFFER_SIZE = 10


class Talker():
    def __init__(self, host, port):

        # ホスト名
        self.host = host
        # ポート番号
        self.port = port

        # 録音して送信するデータを格納する配列
        self.rec_frames = Queue()

        # 受信して再生するデータを格納する配列
        self.play_frames = Queue()

        # ソケットモジュール UDP通信
        self.udp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        # 終了フラグ
        self.finflag = False

        # 録音用ストリーム
        self.stream = pyaudio.PyAudio().open(format = FORMAT,
                            channels = CHANNELS,
                            rate = RATE,
                            input = True,
                            output = True,
                            frames_per_buffer = CHUNK,
                            )


    def start(self):
        """ メインの動作を記述
        録音・再生・受信・送信を並列で行う．
        """

        # バインドする
        self.udp.bind(('', self.port))

        # 終了フラグを初期化
        self.fin_flag = False

        # プロセスを作成
        self.jobs = [
            Process(target=self.record),
            Process(target=self.send),
            Process(target=self.receive),
            Process(target=self.play),
            ]

        # プロセスを開始
        for j in self.jobs:
            j.daemon = True
            j.start()

    def stop(self):
        for j in self.jobs:
            j.terminate()

        for j in self.jobs:
            j.join()

        print('finished')

    def record(self):
        while True:
            self.rec_frames.put(self.stream.read(CHUNK))

    def send(self):
        while True:
            self.udp.sendto(self.rec_frames.get(), (self.host, self.port))


    def receive(self):
        while True:
            # データを受信して再生待ちキューに追加
            soundData, addr = self.udp.recvfrom(CHUNK*2)
            self.play_frames.put(soundData)


    def play(self):
        while not(self.fin_flag):
            # もし受信済みデータが存在するならば先頭をポップして再生
            self.stream.write(self.play_frames.get(), CHUNK)


if __name__ == '__main__':
    """ コマンドライン上で実行する際にはコマンドライン引数でホスト名とポート番号を指定 """
    import time
    import sys

    talker = Talker(sys.argv[1], 40007)

    print('start calling')
    talker.call()

    time.sleep(15)

    talker.stop()
    print('stop calling')
