# -*- coding: utf-8 -*-
# tcp_to_raspi.py
#
# Coded by: Koji KITANO <koji.kitano52@gmail.com>,
# Human Systems Laboratory, Department of Systems Science,
# Graduate School of Informatics, Kyoto University.

import json
import socket

class TCP2Raspi():
    def __init__(self, host, port=50007):
        # 接続先のホスト名
        self.host = host

        # 接続先のポート番号
        self.port = port

        # 送信するデータを格納する配列
        self.frames = []

        # サーバであればTrue
        self.server = False

        # 相手側サーバへのconnectを試みて，失敗したら自らサーバになる
        try:
            # ソケットモジュール
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # ソケットのタイムアウトを設定(5秒)
            self.socket.settimeout(10)
            # connectを試みる
            self.socket.connect((self.host, self.port))
            # ブロッキングモードに変更
            self.socket.settimeout(None)

        except:
            # ソケットモジュール
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # サーバを立てる
            self.socket.bind(("", self.port))
            # 待受を可能にする
            self.socket.listen()
            # 接続を待ち受ける．接続されたら通信用のソケットで上書き
            self.socket, addr = self.socket.accept()
            print("Conneted by"+str(addr))
            # サーバであればTrue
            self.server = True


    def exchange_data(self, data_dict):
        """ 相手側のデバイスとデータを交換 """

        # 辞書型のデータをバイナリに変換
        data_bytes = self._dict_to_bytes(data_dict)

        if self.server:
            # データを送信
            new_data_bytes = self.socket.recv(4096)
            # データを受信
            self.socket.send(data_bytes)
        else:
            # データを送信
            self.socket.send(data_bytes)
            # データを受信
            new_data_bytes = self.socket.recv(4096)

        new_data_dict = self._bytes_to_dict(new_data_bytes)

        return new_data_dict


    def _dict_to_bytes(self, data_dict):
        return json.dumps(data_dict).encode('utf-8')

    def _bytes_to_dict(self, data_bytes):
        return json.loads(data_bytes.decode('utf-8'))
